/**
 * Copyright (c) 2005, Paul Tuckey
 * All rights reserved.
 * ====================================================================
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * ====================================================================
 */
package org.tuckey.web.filters.validation;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.apache.tools.ant.util.FileUtils;
import org.tuckey.web.filters.validation.generated.GeneratedValidationReport;

import javax.xml.parsers.ParserConfigurationException;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.List;

/**
 * @author Paul Tuckey
 * @version $Revision: 9 $ $Date: 2006-09-20 20:36:12 +1200 (Wed, 20 Sep 2006) $
 */
public class ValidationHandlerTest extends TestCase {

    private static File BASE_TEST_FILES_DIR = new File("src/test-web");


    public ValidationHandlerTest(String theName) {
        super(theName);
    }

    public static Test suite() {
        return new TestSuite(ValidationHandlerTest.class);
    }

    public void testGetReport() throws IOException, ParserConfigurationException {
        InputStream is = ValidationHandlerTest.class.getResourceAsStream("/org/tuckey/web/filters/validation/docs/sample-fail.html");
        String contents = FileUtils.readFully(new BufferedReader(new InputStreamReader(is)));
        ValidationHandler vh = new ValidationHandler(contents, "/sample-fail.html");
        vh.parse();
        String report = vh.getReportDiv();
        showReport(report);
    }

    public void testGetReportResinParser() throws IOException, ParserConfigurationException, BadXmlParserException {

        InputStream is = ValidationHandlerTest.class.getResourceAsStream("/org/tuckey/web/filters/validation/docs/sample-fail.html");
        String contents = FileUtils.readFully(new BufferedReader(new InputStreamReader(is)));
        ValidationHandler vh = new ValidationHandler(contents, "/sample-fail.html", "com.caucho.xml.parsers.XmlDocumentBuilderFactory");
        vh.parse();
        String report = vh.getReportDiv();
        showReport(report);
    }

    public void testXhtml10Good() throws IOException, ParserConfigurationException {
        validDocTest("test-xhtml-1.0-good.html");
    }

    public void testXhtmlBasic10() throws IOException, ParserConfigurationException {
        validDocTest("test-xhtml-basic-1.0.html");
    }

    public void testXhtml11() throws IOException, ParserConfigurationException {
        validDocTest("test-xhtml-1.1.html");
    }

    public void testXhtmlFrames10() throws IOException, ParserConfigurationException {
        validDocTest("test-xhtml-frames-1.0.html");
    }

    public void testXhtmlStrict10() throws IOException, ParserConfigurationException {
        validDocTest("test-xhtml-strict-1.0.html");
    }

    public void testXhtmlStrictBad() throws IOException, ParserConfigurationException {
        inValidDocTest("xhtml1.0/strict-bad.html");
    }

    private void validDocTest(String file) throws IOException {
        ValidationHandler vh = doValidDocTest(file);
        vh.parse();
        if (!vh.isValid()) {
            showReport(vh.getReportDiv());
        }
        assertTrue(vh.isValid());
    }

    private void showReport(String report) throws IOException {
        File f = new File("current-test.html");
        System.out.println("for report see " + f.getAbsolutePath());
        new BufferedOutputStream(new FileOutputStream(f)).write(report.getBytes());
    }

    private void inValidDocTest(String file) throws IOException {
        ValidationHandler vh = doValidDocTest(file);
        vh.parse();
        showReport(vh.getReportDiv());
        assertFalse(vh.isValid());
    }

    private ValidationHandler doValidDocTest(String file) throws IOException {
        String contents = FileUtils.readFully(new BufferedReader(new FileReader(new File(BASE_TEST_FILES_DIR, file))));
        ValidationHandler vh = new ValidationHandler(contents, file);
        vh.parse();
        return vh;
    }

    public void testReportIsValid() throws IOException {
        ValidationHandler vh = doValidDocTest("xhtml1.0/strict-bad.html");
        String report = vh.getReportDiv();
        // check we got something
        assertTrue(report.length() > 1024);
        ValidationHandler vhReport = new ValidationHandler(report, "xhtml1.0/strict-bad.html");
        vhReport.parse();
        showReport(report);
        assertFalse(vhReport.isValid());
    }


    public void testSimple() throws Exception {
        ValidationHandler validationHandler = new ValidationHandler(xhtmlDotOrg, "/");
        validationHandler.parse();
        checkNoErrors(validationHandler);
        assertTrue("Source should be valid", validationHandler.isValid());
    }

    public void testCustomXmlParser() throws Exception {
        ValidationHandler validationHandler = new ValidationHandler(xhtmlDotOrg, "/",
                "org.apache.xerces.jaxp.DocumentBuilderFactoryImpl");
        validationHandler.parse();
        checkNoErrors(validationHandler);
        assertTrue("Source should be valid", validationHandler.isValid());
    }

    public void testMainDtd() throws Exception {
        ValidationHandler validationHandler = new ValidationHandler(normal, "/");
        validationHandler.parse();
        checkNoErrors(validationHandler);
        assertEquals("-//W3C//DTD XHTML 1.0 Transitional//EN", validationHandler.getDoctype().getPublicId());
        assertEquals("http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd", validationHandler.getDoctype().getSystemId());
    }

    private void checkNoErrors(ValidationHandler validationHandler) {
        List errors = validationHandler.getErrors();
        for (int i = 0; i < errors.size(); i++) {
            LineIssue spe = (LineIssue) errors.get(i);
            System.err.println("err: " + spe.getMessage());
        }
        assertTrue("There should be 0 errors.  There was " + errors.size(), errors.size() == 0);
    }


    public void testCheckForKnownParser() {
        assertEquals("Apache Crimson", GeneratedValidationReport.checkForKnownParser("org.apache.crimson.jaxp.DocumentBuilderFactoryImpl"));
    }


    private String normal = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"\n" +
            "    \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n" +
            "<html xmlns=\"http://www.w3.org/1999/xhtml\">\n" +
            "<head>\n" +
            "<title>Me title love</title>\n" +
            "<meta name=\"nav\" content=\"top.xml\" />\n" +
            "<meta name=\"template\" content=\"standard\" />\n" +
            "</head>\n" +
            "<body>\n" +
            "<p>aaa</p>\n" +
            "</body>\n" +
            "</html>";

    private String xhtmlDotOrg = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"\n" +
            "    \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n" +
            "<html xmlns=\"http://www.w3.org/1999/xhtml\">\n" +
            "<head>\n" +
            "<meta name=\"generator\" content=\"HTML Tidy, see www.w3.org\" />\n" +
            "<title>XHTML.ORG: A source of information about XHTML</title>\n" +
            "<meta\n" +
            "content=\"This site presents information about W3Cs XHTML specification.\"\n" +
            " name=\"DESCRIPTION\" />\n" +
            "</head>\n" +
            "<body>\n" +
            "</body>\n" +
            "</html>";
}
